from qlat_utils.everything cimport *
from .geometry cimport *

cdef extern from "qlat/field.h" namespace "qlat":

    cdef cppclass Field[T]:
        Field()
        void init()
        void init(const Geometry& geo)
        void init(const Geometry& geo, int multiplicity)
        void init(const Field[T]& field)
        const Field[T]& operator=(const Field[T]& field)
        const Geometry& get_geo()

    Vector[T] get_data[T](const Field[T]& x)
    void set_zero[T](Field[T]& x)
    void qswap[T](Field[T]& x, Field[T]& y)

cdef extern from "qlat/selected-field-io.h" namespace "qlat":

    cdef cppclass FieldSelection:
        FieldSelection()
        const Geometry& get_geo()

    cdef cppclass PointsSelection:
        PointsSelection()
        PointsSelection(const long n_points)
        long size()
        Coordinate* data()
        Coordinate& operator[](long i)

    cdef cppclass SelectedField[T]:
        long n_elems;
        SelectedField()
        void init()
        void init(const Geometry& geo, const long n_elems, const int multiplicity)
        void init(const FieldSelection& fsel, const int multiplicity)
        const SelectedField[T]& operator=(const SelectedField[T]& field)
        const Geometry& get_geo()

    Vector[T] get_data[T](const SelectedField[T]& x)
    void set_zero[T](SelectedField[T]& x)
    void qswap[T](SelectedField[T]& x, SelectedField[T]& y)

    cdef cppclass SelectedPoints[T]:
        int multiplicity
        long n_points
        SelectedPoints()
        void init()
        void init(const long n_points, const int multiplicity)
        void init(const PointsSelection& psel, const int multiplicity)
        const SelectedPoints[T]& operator=(const SelectedPoints[T]& field)

    Vector[T] get_data[T](const SelectedPoints[T]& x)
    void set_zero[T](SelectedPoints[T]& x)
    void qswap[T](SelectedPoints[T]& x, SelectedPoints[T]& y)



#    Qlattice (https://github.com/waterret/qlattice)
#
#    Copyright (C) 2022
#
#    Author: Luchang Jin (ljin.luchang@gmail.com)
#    Author: Masaaki Tomii
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from auto_contractor.eval import \
        load_prop, \
        ama_apply, ama_apply1, ama_apply2, ama_apply2_r, ama_apply2_l, ama_list, ama_counts, ama_extract

from qlat_utils import \
        rel_mod, rel_mod_sym, c_rel_mod_sqr

from qlat import \
        timer, timer_flops

import numpy
